@@ -8,16 +8,15 @@ from sql.aggregate import Max, Count, Sum
 from sql.conditionals import Case
 from sql.functions import Extract
 
-from trytond.model import ModelView, ModelSQL, Workflow, fields, Check
-from trytond.wizard import Wizard, StateView, StateAction, Button
+from trytond.model import ModelView, ModelSQL, Workflow, fields, \
+    sequence_ordered
 from trytond import backend
-from trytond.pyson import Eval, In, If, Get, PYSONEncoder
+from trytond.pyson import Eval, In, If, Get, Bool
 from trytond.transaction import Transaction
 from trytond.pool import Pool
 
 __all__ = ['SaleOpportunity', 'SaleOpportunityLine',
-    'SaleOpportunityEmployee',
-    'OpenSaleOpportunityEmployeeStart', 'OpenSaleOpportunityEmployee',
+    'SaleOpportunityEmployee', 'SaleOpportunityEmployeeContext',
     'SaleOpportunityMonthly', 'SaleOpportunityEmployeeMonthly']
 
 STATES = [
@@ -87,7 +86,12 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
         states=_STATES_STOP, depends=_DEPENDS_STOP)
     state = fields.Selection(STATES, 'State', required=True, select=True,
             sort=False, readonly=True)
-    probability = fields.Integer('Conversion Probability', required=True,
+    conversion_probability = fields.Float('Conversion Probability',
+        digits=(1, 4), required=True,
+        domain=[
+            ('conversion_probability', '>=', 0),
+            ('conversion_probability', '<=', 1),
+            ],
         states={
             'readonly': ~Eval('state').in_(
                 ['opportunity', 'lead', 'converted']),
@@ -140,16 +144,18 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
                         where=sale.id == sale_id))
             table.drop_column('sale', exception=True)
 
+        # Migration from 4.0: change probability into conversion probability
+        if table.column_exist('probability'):
+            cursor.execute(*sql_table.update(
+                    [sql_table.conversion_probability],
+                    [sql_table.probability / 100.0]))
+            table.drop_constraint('check_percentage')
+            table.drop_column('probability')
+
     @classmethod
     def __setup__(cls):
         super(SaleOpportunity, cls).__setup__()
         cls._order.insert(0, ('start_date', 'DESC'))
-        t = cls.__table__()
-        cls._sql_constraints += [
-            ('check_percentage',
-                Check(t, (t.probability >= 0) & (t.probability <= 100)),
-                'Probability must be between 0 and 100.')
-            ]
         cls._error_messages.update({
                 'delete_cancel': ('Sale Opportunity "%s" must be cancelled '
                     'before deletion.'),
@@ -201,8 +207,8 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
         return Date.today()
 
     @staticmethod
-    def default_probability():
-        return 50
+    def default_conversion_probability():
+        return 0.5
 
     @staticmethod
     def default_company():
@@ -210,14 +216,7 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
 
     @staticmethod
     def default_employee():
-        User = Pool().get('res.user')
-
-        if Transaction().context.get('employee'):
-            return Transaction().context['employee']
-        else:
-            user = User(Transaction().user)
-            if user.employee:
-                return user.employee.id
+        return Transaction().context.get('employee')
 
     @classmethod
     def default_payment_term(cls):
@@ -426,25 +425,36 @@ class SaleOpportunity(Workflow, ModelSQL, ModelView):
             cls.convert(converted)
 
 
-class SaleOpportunityLine(ModelSQL, ModelView):
+class SaleOpportunityLine(sequence_ordered(), ModelSQL, ModelView):
     'Sale Opportunity Line'
     __name__ = "sale.opportunity.line"
     _rec_name = "product"
     _history = True
-    opportunity = fields.Many2One('sale.opportunity', 'Opportunity')
-    sequence = fields.Integer('Sequence')
+    _states = {
+        'readonly': Eval('opportunity_state').in_(
+            ['converted', 'won', 'lost', 'cancelled']),
+        }
+    _depends = ['opportunity_state']
+
+    opportunity = fields.Many2One('sale.opportunity', 'Opportunity',
+        states={
+            'readonly': _states['readonly'] & Bool(Eval('opportunity')),
+            },
+        depends=_depends)
+    opportunity_state = fields.Function(
+        fields.Selection(STATES, 'Opportunity State'),
+        'on_change_with_opportunity_state')
     product = fields.Many2One('product.product', 'Product', required=True,
-        domain=[('salable', '=', True)])
+        domain=[('salable', '=', True)], states=_states, depends=_depends)
     quantity = fields.Float('Quantity', required=True,
-            digits=(16, Eval('unit_digits', 2)), depends=['unit_digits'])
-    unit = fields.Many2One('product.uom', 'Unit', required=True)
+        digits=(16, Eval('unit_digits', 2)),
+        states=_states, depends=['unit_digits'] + _depends)
+    unit = fields.Many2One('product.uom', 'Unit', required=True,
+        states=_states, depends=_depends)
     unit_digits = fields.Function(fields.Integer('Unit Digits'),
         'on_change_with_unit_digits')
 
-    @classmethod
-    def __setup__(cls):
-        super(SaleOpportunityLine, cls).__setup__()
-        cls._order.insert(0, ('sequence', 'ASC'))
+    del _states, _depends
 
     @classmethod
     def __register__(cls, module_name):
@@ -456,10 +466,10 @@ class SaleOpportunityLine(ModelSQL, ModelView):
         # Migration from 2.4: drop required on sequence
         table.not_null_action('sequence', action='remove')
 
-    @staticmethod
-    def order_sequence(tables):
-        table, _ = tables[None]
-        return [Case((table.sequence == Null, 0), else_=1), table.sequence]
+    @fields.depends('opportunity', '_parent_opportunity.state')
+    def on_change_with_opportunity_state(self, name=None):
+        if self.opportunity:
+            return self.opportunity.state
 
     @fields.depends('unit')
     def on_change_with_unit_digits(self, name=None):
@@ -473,8 +483,7 @@ class SaleOpportunityLine(ModelSQL, ModelView):
             return
 
         category = self.product.sale_uom.category
-        if (not self.unit
-                or self.unit not in category.uoms):
+        if not self.unit or self.unit.category != category:
             self.unit = self.product.sale_uom
             self.unit_digits = self.product.sale_uom.digits
 
@@ -615,31 +624,13 @@ class SaleOpportunityEmployee(SaleOpportunityReportMixin, ModelSQL, ModelView):
         return query
 
 
-class OpenSaleOpportunityEmployeeStart(ModelView):
-    'Open Sale Opportunity per Employee'
-    __name__ = 'sale.opportunity_employee.open.start'
+class SaleOpportunityEmployeeContext(ModelView):
+    'Sale Opportunity per Employee Context'
+    __name__ = 'sale.opportunity_employee.context'
     start_date = fields.Date('Start Date')
     end_date = fields.Date('End Date')
 
 
-class OpenSaleOpportunityEmployee(Wizard):
-    'Open Sale Opportunity per Employee'
-    __name__ = 'sale.opportunity_employee.open'
-    start = StateView('sale.opportunity_employee.open.start',
-        'sale_opportunity.opportunity_employee_open_start_view_form', [
-            Button('Cancel', 'end', 'tryton-cancel'),
-            Button('Open', 'open_', 'tryton-ok', default=True),
-            ])
-    open_ = StateAction('sale_opportunity.act_opportunity_employee_form')
-
-    def do_open_(self, action):
-        action['pyson_context'] = PYSONEncoder().encode({
-                'start_date': self.start.start_date,
-                'end_date': self.start.end_date,
-                })
-        return action, {}
-
-
 class SaleOpportunityMonthly(SaleOpportunityReportMixin, ModelSQL, ModelView):
     'Sale Opportunity per Month'
     __name__ = 'sale.opportunity_monthly'
